
# Proposed Changes to ETSI GS QKD 004 Version 2.1.1 (2020-08)


## Clarifications

### NULL KSID

A KSID (`Key_stream_ID`) is referred to as "NULL" if it is comprised
of all-zero bits, that is, if all 128 bits of its UUID representation are zero.

If a client passes a NULL KSID during an OPEN_CONNECT call, it requests the
server to select a random, unused KSID and to return this non-NULL KSID in its
OPEN_CONNECT response.

### NULL Index

An index parameter is referred to as "NULL" if all of its 32 bits are zero.

If a client passes a NULL index during a GET_KEY call, it requests the
server to pick the lowest non-NULL index value for which key material is
available or will become available.  The server returns the respective
non-NULL index value in its GET_KEY response.  Note that the server may
indicate that sufficient key material is not yet available.

### NULL QoS Parameter

An integer QoS parameter is referred to as "NULL" if all its 32 bits are zero.

A char or byte array QoS parameter is referred to as "NULL" if its first byte
is zero.

### NULL URI

A URI is referred to as "NULL" if its first byte is zero.

A NULL URI is interpreted as "unspecified URI".

See _SAE IDs and URIs_ for more information.

### Key_buffer

For the avoidance of doubt, please note that the term "Key_buffer" as used in
the ETSI GS QKD 004 specification refers to the part of the server's GET_KEY
response structure holding the data of the single key delivered in response
to the client's GET_KEY request.  The status parameter of the server's response
needs to be consulted to determine whether the respective bytes indeed
represent a valid key.

### GET_KEY Calls with Non-Consecutive Indices

Clients are not required to access keys with consecutive indices.
For example, it is generally permissible to issue a GET_KEY request
for index=1 after a GET_KEY request for index=2 was issued.  The server
is required to honor GET_KEY requests with non-consecutive indices as
defined by its `Capacity` QoS guarantee.

See _Jitter becomes Capacity_ and _Index out of Bounds_ below for more
information.

### KME-KME and KME-Controller Interfaces

The ETSI GS QKD 004 specification and its reference implementation
deal with a client-side key retrieval interface and the underlying
client-server (SAE-KME) protocol.

In order for KMEs to implement the functionality required to provide
an ETSI GS QKD 004 compliant service, additional KME-KME and
KME-Controller interfaces are required.  At present, the details of
these interfaces are outside the scope of ETSI QKD 004.


## TLS Connection Handling

### Basic Rules

1) The ETSI GS QKD 004 reference implementation allows multiple commands per
   TLS connection, e.g., enabling a client to issue an OPEN_CONNECT, a couple
   of GET_KEY commands, and a CLOSE using the same TLS connection.
2) It is, however, not required that a client uses the same TLS connection
   for all commands pertaining to a given KSID.  A client can close a TLS
   connection and re-open a different one at a later point without losing
   access to the KSID.
3) An ETSI GS QKD 004 client-server TLS connection is not bound to a KSID.
   That is, a client may submit commands pertaining to different KSIDs using
   a given TSL connection to a server.
4) In the context of a given TLS connection, commands are blocking.  That is,
   a client must wait for the complete response of a server prior to submitting
   another command.

### Server-side Timeout

A server is allowed to close an "inactive" TLS connection.  Neither the ETSI
GS QKS 004 standard nor the ETSI GS QKD reference implementation prescribes
a means to configure when, i.e., after which period of inactivity, a server
is allowed to consider a TLS connection inactive and, hence, close it.

That being said, a server must not close a connection with pending commands.

The documentation of a server implementation should indicate under which
conditions a server will close a TLS connection.


## SAE IDs and URIs

### Format

The format of SAE IDs and KME IDs needs to be aligned with other
relevant ETSI GS QKD standards such as ETSI GS QKD 014.  The current
ETSI GS QKD 004 reference implementation is written under the working
hypothesis that both SAE IDs and KME IDs will be specified as URIs.
Further details will have to be jointly agreed upon by the respective
ETSI GS QKD working groups.

For KME IDs, we suggest the following format in accordance with [RFC
3986](https://www.rfc-editor.org/rfc/rfc3986) and [RFC
8820](https://www.rfc-editor.org/rfc/rfc8820):

- Only the `authority` component of the URI is used.
- The `scheme`, `path`, `query`, and `fragment` components remain
  unused and are reserved for future extensions.
- The optional `userinfo` subcomponent of `authority` remains unused
  and is reserved for future extensions.
- We follow RFC 3986 regarding the `host` and `port` subcomponents of
  `authority`.
    * Specification of `host` is mandatory.
    * `host` can either be a hostname or an IP address.
    * Specification of `port` is optional.
      (We assume a default port number to be defined in the respective
      standards.)

### New Destination KME ID Parameter in OPEN_CONNECT Call

In order to address multi-hop scenarios (quantum key distribution via trusted
nodes), a Destination_KME_ID parameter is added to the OPEN_CONNECT call.  This
allows for the source KME, i.e., the server handling the OPEN_CONNECT call, to
take measures aimed at determining a route to the destination KME.

In most cases, the source KME is expected to contact a QKD controller to
obtain a routing solution.  There may be exceptions in case the source KME is
directly connected to the destination KME via a single QKD link.  Details of this
process are outside the scope of the ETSI GS QKD 004 specification.

Giving a NULL destination KME ID as parameter of an OPEN_CONNECT that
establishes a key stream (a.k.a. "first OPEN_CONNECT call", cf. _Behavior
of First and Second OPEN_CONNECT_ below) is permissible in two cases:

1) In case the source KME, i.e., the server handling the OPEN_CONNECT
   call, is connected to exactly one destination KME.
2) If a non-NULL destination SAE ID was specified _and_ the source KME
   is directly or indirectly (via its QKDN controller) connected to a
   trusted SAE discovery service.  This trust service maps destination
   SAE IDs to destination KME IDs.

See the sections entitled _SAE Discovery_, _SAE Binding of KSIDs_, and
_Summary of SAE/KME ID Options_ below for more information.

### Source and Destination SAE IDs in OPEN_CONNECT Call

#### Naming

In order to better align with the ETSI GS QKD 014 specification and
other relevant ETSI GS QKD standards, we will use "source/destination
SAE ID" instead of "source/destination URI" or "source/destination
identifier" going forward.

#### Optional Meta-Information

Source and destination SAE IDs are optional pieces of meta-information
and, hence, can be left unspecified. I.e., the client is allowed to
put NULL URIs for source and/or destination SAE ID in an OPEN_CONNECT
call.

Source and destination SAE IDs are not required for use cases 1-4 as
described in V2.1.1 ETSI GS QKD 004 document.  See the section
entitled _SAE discovery_ for more information regarding _Application
discovery in a QKD network_ (use case 5).

Specifying a destination SAE ID can also be used to enforce SAE
binding of KSIDs.  See the sections _SAE Binding of KSIDs_ and
_Summary of SAE/KME ID Options_ below for more information.

### SAE Discovery

The QKD ecosystem requires an efficient, easy-to-use KME interface
that is geared towards certifiable lightweight implementations.
Adding too much weight would be counterproductive.  Instead, ETSI GS
QKD 004 strives to optionally provide the necessary meta-information
for higher-layer functions to provide value-added services.  SAE
Discovery is an example of a such a service.

In some QKD networks it may be desirable to offer a trusted SAE
discovery service.  Via the optional destination SAE ID in the
OPEN_CONNECT call, this version of the ETSI GS QKD 004 specification
provides the necessary pieces of information to support such a flow.
However, ETSI GS QKD 004 neither requires such a service to be
available nor does it specify details of its implementation.

SAE Discovery trust services, i.e., trust services mapping SAE IDs to
KME IDs, can be located in both the source KME and its QKD controller.
If a KME does not have a SAE Discovery service or if its SAE Discovery
service cannot resolve the destination SAE ID then it can either
forward the SAE-KME mapping query to the QKD controller or reject the
respective OPEN_CONNECT request.

Details of implementation and operational aspects of an SAE discovery
service are best defined in the context of specific use cases and
their respective legislation, regulations, and certification
requirements.  This will then address topics such as

- details of the PKI trust services (such as RA, CA, OCSP responder,
  certificate validity, cryptographic algorithms and key sizes,
  handling of root certificates, and so on) providing trusted SAE ID
  certificates;
- details of PKI trust services providing KME ID certificates for
  mutual SAE-KME authentication; and
- details of trusted SAE-KME ID mapping services including trust
  anchors and high-availability/fail-over requirements.

Note that SAE Discovery services do not have to be PKI based.  In
theory, authenticated administrators of KME and/or QKDN controller
could load (static) SAE-KME mapping tables.  Trust would then
hinge on authentication and operational processes.  Again, details
have to be defined in the context of specific use cases and their
respective legislation, regulations, and certification requirements.

See _Summary of SAE/KME ID Options_ below for more information.

### SAE Binding of KSIDs

Another higher-layer functionality that is enabled (yet not required)
by ETSI GS QKD 004 is _binding of a KSID to a given SAE ID_.  An
initiator SAE may wish to instruct the QKD network that access to a
newly created key stream is to be restricted to the SAE with the given
source and destination SAE ID (optional parameter of the OPEN_CONNECT
call; see above).

Again, specific requirements of an SAE-KSID binding functionality,
e.g., SAE authentication requirements, are outside the scope of the
ETSI GS QKD 004 specification, and are best defined in the context of
specific use cases and their respective legislation, regulations, and
certifications.

See _Summary of SAE/KME ID Options_ below for more information.

### Summary of SAE/KME ID Options

The following provides a summary of the different cases.

1) Non-NULL KME ID was specified in OPEN_CONNECT
    * Source KME and/or its QKDN controller determine route to
      destination KME.  OPEN_CONNECT request must be rejected if the
      destination KME is unknown or no route can be determined.
    * SAE discovery is not used.
    * Specifying only one SAE ID (either source or destination) is a
      protocol violation.  The source KME must reject the respective
      OPEN_CONNECT request.
2) NULL KME ID was specified in OPEN_CONNECT
    a) NULL destination SAE was specified
        - This is valid if and only if the source KME is connected to
          exactly one destination KME.  Else, the source KME must reject
          the OPEN_CONNECT request.
        - SAE discovery is not used.
        - SAE-KSID binding is not used.
    b) Non-NULL destination SAE was specified
        - Source KME and/or its QKDN controller must be connected to a
          trusted SAE discovery service that maps the destination SAE ID to
          a destination KME ID, or else the source KME must reject the
          OPEN_CONNECT request.
        - If the SAE discovery service cannot resolve the destination KME ID,
          i.e., it does not return a destination KME ID, or if the returned
          destination KME ID is unknown to the source KME and its controller
          then the source KME has to reject the OPEN_CONNECT request.
        - Source KME and/or its QKDN controller determine route to the
          destination KME.  The OPEN_CONNECT request must be rejected if no
          route can be determined.

In all cases the following holds:

* SAE-KSID binding is only performed if non-NULL SAE IDs are given for _both_
  source and destination SAE.
* If a KME cannot perform a requested (source or destination) SAE-KSID
  binding then it must reject the OPEN_CONNECT request.


## Authentication

### Authentication Requirements in the ETSI GS QKD 004 Standard

The ETSI GS QKD 004 standard does not _require_ authentication between
client (SAE) and server (KME).  However, authentication can be layered
on top of specific ETSI GS QKD 004 implementations.  This is a
deliberate choice in order to render ETSI GS QKD 004 usable in
environments that

- either do not require cryptographic authentication (a few),
- or that have very specific authentication needs, e.g., due to
  regulatory requirements (the vast majority).

As ETSI GS QKD 004 cannot prescribe a canonical one-size-fits-all
approach to authentication, it is a deliberate design choice to treat
authentication as an orthogonal, independent subject.

The next section will show that care has been taken to not mandate or
exclude specific authentication options in the context of the ETSI GS
QKD 004 reference implementation.

### Authentication in the ETSI GS QKD 004 Reference Implementation

The ETSI GS QKD 004 reference implementation uses TLS for client-server
communication.  A number of best practices are enforced in the source code.
These include

- the use of TLS 1.3,
- a highly secure ciphersuite (`TLS_AES_256_GCM_SHA384`), and
- provisions disabling TLS session resumption for security reasons.

Some other best practices are recommended but not enforced.
These include

- the use of strong RSA keys,
- the use of pre-shared keys (PSKs) in addition to an (EC)DHE key
  exchange, and
- considering the use of PQC extensions to TLS.

See [README.md](README.md) for details.

The ETSI GS QKD 004 reference implementation does not mandate specific
authentication requirements though.

- The client and server examples show how to enable and disable client
  and server certification verification.
- Even with client and/or server certificate verification enabled,
  there is no hard-coded requirement regarding PKI trust services.
  Users of the reference implementation have full control over which
  self-signed root CA certificates they load.


## Changes to QoS Parameters

We strive to make ETSI GS QKD 004 easier to use while maintaining its
expressive power.  This is also reflected in some simplifications and
improvements of the QoS parameters supported by default.

Please note that clients and servers can exchange custom information
via the metadata parameters of the OPEN_CONNECT and GET_KEY commands.

### New OPEN_CONNECT Metadata Parameter

In order to allow for the transport of additional pieces of
information that are required to implement higher-layer services, an
`inout Metadata` parameter has been added to the OPEN_CONNECT command.
A corresponding `OC_metadata_mimetype` has been added to the
OPEN_CONNECT QoS parameter structure (here, "OC" stands for
OPEN_CONNECT).  For reasons of consistency, the pre-existing mimetype
field will be renamed to `GK_metadata_mimetype`.

One application area for OPEN_CONNECT metadata is priority and
accounting information.  See _Priority moved to Metadata_ below for
more information.

### TTL becomes TTF

#### Time To Fetch

We renamed the TTL parameter (time to live) to TTF (time to fetch)
along with a more-precise and measurable definition of its meaning.

In a given key stream, each key has a unique index.  A key with a
given index is said to have _entered the key stream_ when the first
SAE (this could be either the initiator SAE or the destination SAE)
has been delivered the key in response to a GET_KEY command.  At this
point, this key's TTF clock starts ticking.  The second SAE (the one
that has not yet consumed the key with the given index) must consume
the key within TTF seconds of it entering the key stream.  Else,
trying to access the key with the given index may result in an error
code.  See _Status Codes_ below for more information.

In summary, the TTF parameter can serve a number of different
purposes.

1) It serve as a security requirement enforcing a timely consumption
   of a key that has entered a key stream.
2) It can serve as a lower bound for the remaining TTL (time to live).
   See _Time To Live_ below for more information.
3) It can help to implement a self-healing mechanism allowing a KME to
   get rid of _pending keys_, i.e., keys that have entered the
   key stream but have not yet been picked up by a the second SAE.
   (Something like this can happen in certain cases if SAE-SAE
   messages get lost.  If the KME was not allowed to get rid of
   pending keys after a reasonable time to fetch then the
   effective remaining capacity allocated to the respective key stream
   would shrink over time, even if a sparse memory implementation was
   used.)

#### Time To Live

It may still make sense to define a TTL parameter, albeit outside of
the scope of ETSI GS QKD 004.  Such a TTL parameter could be related
to some sort of best-before time of a QKD key, e.g.,

- the time between the generation of a key by a QKD link device and
  its consumption by an SAE or
- the time between a KME internally assigning a particular key to a
  KSID (this may happen well before the GET_KEY response finally
  delivering the key) and its consumption by the connected SAEs.

Note that in both cases an SAE cannot detect when the TTL clock starts
ticking for any given key.

The definition of such TTL parameters falls into the realm of security
evaluations of QKD link devices and KMEs, and should be addressed
there.

TTL parameters of QKD link devices and KMEs could potentially
invalidate a key before its TTF clock expires.  In order to avoid
that, a KME delivering a key to a key stream has to ensure the
remaining (!) TTL is at least as long as the TTF.

#### Key Stream Lifetime

At present, there is no QoS parameter indicating key stream lifetime.
That is, it is not defined exactly how long a server will have to keep
a key stream open if there are no _pending keys_.

Keys are said to be _pending_ if only one endpoint has fetched the key
with a GET_KEY command and the server has not yet deleted the key due
to a TTL or TTF timeout.

It is reasonable to request that a key stream be kept open long enough
for a burst of size `Capacity` to succeed.  Filling up the respective
server-side key store would take `L = Capacity * 8 * Key_chunk_size /
Min_bps` seconds.

We therefore suggest the following rules.

1) A server must keep a key stream accessible if there a pending keys.
2) A server must keep a key stream open for at least `2 * L` seconds
   if there are no pending keys.
3) A server can keep a key stream alive for a longer period.
4) The documentation of a key server should describe its key stream
   Lifetime policy.

### Jitter becomes Capacity

We replaced the `Jitter` QoS parameter by `Capacity` for a number of
reasons including

- better support for bursty key consumption patterns,
- ease of configuration, and
- ease of checking whether a server does fulfill its QoS guarantees.

The new 32-bit integer `Capacity` parameter is defined as the number
of keys with consecutive indices that can be retrieved in a burst from
the server if the respective KSID's server-side key store is full.

The server is required to provide at least `Capacity` times
`Key_chunk_size` bytes of server-side key storage for the KSID.

The server is not required to implement sparse memory for key storage.
See _Index out of Bounds_ below for more information.

For the avoidance of doubt: The `Capacity` and `Min_bps` parameters
can be configured independently. (To recap: The `Min_bps` defines the
minimum key generation rate a server guarantees a client.)

### Max_bps

The Max_bps QoS parameter has a more well-defined meaning in the
presence of a Capacity QoS parameter.  Using Max_bps a client can
indicate at which (burst) rate it will access key material of the
respective key stream that is already available in the server-side key
store.

Hence, another way to look at Min_bps and Max_bps is:

- Min_bps is the guaranteed rate with which the key stream's
  server-side key store (capable of holding at least Capacity keys of
  Key_chunk_size bytes) is being filled.
- Max_bps is the guaranteed rate with which a client can access the
  key stream's server-side key store.

A server must reject the request if it cannot guarantee the requested
burst rate.

### Priority moved to Metadata

The `Priority` QoS parameter of ETSI GS QKD 004 V2.1.1 had no
well-defined semantics.  Hence, information related to topics such as
priority and accounting is moved into the (new) realm of OPEN_CONNECT
metadata.


## Behavior of First and Second OPEN_CONNECT

Both SAEs have to issue an OPEN_CONNECT call.  In some situations
there may be clear _initiator SAE_ and _target SAE_ roles such that
the initiator SAE always issues the first OPEN_CONNECT.  In other
cases, where the relationship between the SAEs may be closer to peers,
the sequence of OPEN_CONNECT calls may be random.

To process an OPEN_CONNECT call, KMEs need to communicate horizontally
with their peers and/or vertically with their QKDN controller(s).
Even if two OPEN_CONNECT calls reached their respective KMEs at the
very same point in time, at the level of KME-KME or KME-controller
communication handling of these OPEN_CONNECT calls will (have to)
happen in sequence.  Hence, there will always be a _first OPEN_CONNECT
call_ and a _second OPEN_CONNECT_ call.

This section will define requirements regarding SAE-level behavior of
first and second OPEN_CONNECT calls.

### NULL QoS Parameters

For the following QoS parameters

- Max_bps,
- Min_bps,
- Capacity,
- Timeout, and
- TTF

passing a NULL value (see _NULL QoS Parameter_ above) shall mean
"don't care".

Passing a NULL value for Key_chunk_size shall result in an error
(Status = 7) with the KME suggesting a non-NULL value in its response.

#### Null QoS Parameters in First OPEN_CONNECT Call

A KME shall respond with meaningful (default) values in response to
such a NULL QoS parameter.  In particular, the KME must return
non-NULL values for all of these QoS parameters.

#### Null QoS Parameters in Second OPEN_CONNECT Call

A KME shall respond with the values established in the course of
processing the first OPEN_CONNECT call to such a NULL QoS parameter.

### Non-NULL QoS Parameters in Second OPEN_CONNECT Call

If an SAE sends non-NULL values for any for the following QoS
parameters during a second OPEN_CONNECT call

- Max_bps,
- Min_bps,
- Capacity,
- Timeout, and
- TTF

then the KME shall return an error code (Status = 7) if there is a
mismatch between the value given and the corresponding value
established in the course of processing the first OPEN_CONNECT call.
In this case, the KME shall return the QoS settings established as a
result of processing the first OPEN_CONNECT call.

### SAE IDs

The SAE IDs given during the first OPEN_CONNECT call determine whether
SAE Discovery and SAE-KSID binding are used.

The SAE IDs given during the second OPEN_CONNECT call

- must not constitute a protocol violation (cf. _Summary of SAE/KME ID
  Options_) and
- must not contradict the SAE IDs given during the first OPEN_CONNECT
  call.  In particular, if a non-NULL SAE ID given as parameter of the
  second OPEN_CONNECT call does not match the respective SAE ID of the
  first OPEN_CONNECT call (source and destination must be flipped, of
  course) then the KME has to reject the request.
- It is permissible to specify NULL SAE IDs during the second
  OPEN_CONNECT call without affecting SAE Discovery or SAE-KSID
  binding.

### KME ID Parameter in Second OPEN_CONNECT Call

The KME ID parameter of the second OPEN_CONNECT call can be NULL even if a
non-NULL KME ID was used in the first OPEN_CONNECT call.

If a non-NULL KME ID is given during the second OPEN_CONNECT call then it must
match the destination KME ID, i.e., it must be identical to the source KME ID
of the first OPEN_CONNECT call.

### KSID Parameter in Second OPEN_CONNECT Call

The KSID parameter of the second OPEN_CONNECT call must match the KSID of the
key stream established as a result of the first OPEN_CONNECT call.

Note that giving a NULL KSID as parameter of an OPEN_CONNECT call will render
it as a _first_ OPEN_CONNECT call.

## Status Codes

### New Error Codes in a Nutshell

A number of new (32 bit integer) status codes have been introduced.

9: Call failed due to memory allocation problems.

10: GET_KEY failed as the index is out of bounds.  See below for
    details.

11: GET_KEY failed because the key was either already consumed (i.e.,
    the caller received the key with the very index in response to an
    earlier GET_KEY call) or because it was destroyed due to a TTF
    timeout.  Either way, the key has been removed from the server and
    can not longer be retrieved by the client.

12: OPEN_CONNECT failed because no route to destination could be found.

13: OPEN_CONNECT failed because no application discovery service was available.

14: OPEN_CONNECT failed because of a protocol violation.

0x80000000: Error codes starting with 0x80000000 (MSB of the status
    code set) are considered _vendor defined error codes_.

### Index out of Bounds

An index is considered to be _out of bounds_ if it points too far into
the future.  Recall that a server is required to allocate at least
enough storage space for `Capacity` keys with consecutive indices.

Let `N` be the smallest index of unconsumed keys (i.e., neither
consumed by both clients in response to GET_KEY calls nor deleted by
the server due to a TTF or TTL timeout) of the given KSID.  A server
can then respond with _index out of bounds_ to any GET_KEY request
with an index of `N + Capacity` or higher.

Please note

- The server is not required to implement sparse memory.  Even if the
  keys with indices `N+1` to `N+Capacity-1` were all consumed then a
  server could still respond with _index out of bounds_ to a GET_KEY
  request with index `N+Capacity`.
- Keys are considered to be _not yet consumed_ if they are in
  pending state.  Hence, a "slow" SAE, i.e., an SAE consuming
  keys at a relatively low rate, may have a negative impact on the
  effectively available capacity as experienced by its "faster" peer
  SAE.


## Layering ETSI GS QKD 014 on Top of ETSI GS QKD 004

The proposed changes to the ETSI GS QKD 004 specification and its corresponding
reference implementation have been carefully crafted to allow for layering
an ETSI QKD 014 API on top of ETSI GS QKD 004.