#!/bin/bash

# Build and start a dev container for the ETSI QKD 004 reference implementation

# Choose your container manager
# CONTAINER_MANAGER=docker
CONTAINER_MANAGER=podman

# The directory where this script is located
SCRIPT_DIR=`dirname $0`

# The container image name
IMAGE=etsi-004-builder

# The container name (same as the image name here)
CONTAINER=${IMAGE}

# The working directory in the container
WORK_DIR=/workspace/builder/

# Build an image, then create a container from the image, then start the container

# Build the image if it does not exist already
 if [ -z "`podman images -n ${IMAGE}`" ]; then
    echo Building image ${IMAGE}
    ${CONTAINER_MANAGER} build ${SCRIPT_DIR} -t ${IMAGE}
 fi

# Create the container if it does not exist already
if [ -z "`${CONTAINER_MANAGER} container ls --all --noheading --filter name=^${CONTAINER}\$`" ]; then
    echo Creating container ${CONTAINER}
    ${CONTAINER_MANAGER} create --name ${CONTAINER} -t -v${SCRIPT_DIR}:${WORK_DIR} -w${WORK_DIR}/etsi-gs-qkd-004-c ${IMAGE}
fi

# Start the container if it is not yet running
if [ -z "`${CONTAINER_MANAGER} ps --noheading --filter name=^${CONTAINER}\$`" ]; then
    echo Starting container ${CONTAINER}
    ${CONTAINER_MANAGER} start ${CONTAINER}
fi

# Uncomment the following line if you want to see your running (!) containers
#${CONTAINER_MANAGER} container ls

# Uncomment the following line if you want to log into the container
#${CONTAINER_MANAGER} exec -it ${CONTAINER} bash
