#[=======================================================================[.rst:
FindUuid
--------

Finds the UUID library.


Imported Targets
^^^^^^^^^^^^^^^^

This module defines the :prop_tgt:`IMPORTED` target ``Uuid::uuid``, if
the UUID library has been found.


Result Variables
^^^^^^^^^^^^^^^^

This module defines the following variables:

``Uuid_FOUND``
  True if Uuid_INCLUDE_DIR & Uuid_LIBRARY are found.

``Uuid_LIBRARIES``
  List of libraries when using the UUID library.

``Uuid_INCLUDE_DIRS``
  Where to find the UUID headers.


Cache variables
^^^^^^^^^^^^^^^

The following cache variables may also be set:

``Uuid_INCLUDE_DIR``
  The UUID include directory.

``Uuid_LIBRARY``
  The absolute path of the UUID library.
#]=======================================================================]

# Find the library.
find_path(Uuid_INCLUDE_DIR NAMES uuid/uuid.h
    DOC "The UUID include directory"
    )
find_library(Uuid_LIBRARY NAMES uuid
    DOC "The UUID library"
    )

# Check that all required variables are found.
# This also sets Uuid_FOUND, and reports failure if a required variable
# could not be found.
include(FindPackageHandleStandardArgs)
find_package_handle_standard_args(Uuid
    DEFAULT_MSG
    Uuid_LIBRARY Uuid_INCLUDE_DIR
    )

if (Uuid_FOUND)
    set(Uuid_INCLUDE_DIRS ${Uuid_INCLUDE_DIR})
    set(Uuid_LIBRARIES ${Uuid_LIBRARY})

    add_library(Uuid::uuid UNKNOWN IMPORTED)
    set_target_properties(Uuid::uuid PROPERTIES
        IMPORTED_LOCATION "${Uuid_LIBRARIES}"
        INTERFACE_INCLUDE_DIRECTORIES "${Uuid_INCLUDE_DIRS}")
endif ()
