#!/bin/bash

# We set max_num_KSIDs of the server to 3 (via a config file) and
# then we try to connect 4 clients.  We need all clients to have
# a different effective KSID.  Hence, we set client_uuid_string to
# an all-zeroes value in the client's configuration file.  The server
# will then generate random KSIDs for the clients.
# To let the clients linger a bit, we make them wait a while before
# they issue their second GET_KEY request.

test_dir=`dirname ${BASH_SOURCE[0]}`
test_name="too_many_KSIDs"

# valgrind helpers (active if ${RUN_TESTS_UNDER_VALGRIND}=1)
. ${test_dir}/../../.valgrind

# Remove lingering semaphores, if any.
env QKD_SERVER_FORCE_SEM_UNLINK=1 ./server_example 1001 A

# The actual test starts here.
timeout 12s ${memcheck1} ./server_example 1001 A ${test_dir}/max_three_KSIDs.cfg &

# Give the server 2s to start up and listen on his port.
sleep 2

./client_example ${test_dir}/zero_client_uuid.cfg 0 0 4 0 2>&1 | grep Status > ${test_name}.1 &
./client_example ${test_dir}/zero_client_uuid.cfg 0 0 4 0 2>&1 | grep Status > ${test_name}.2 &
./client_example ${test_dir}/zero_client_uuid.cfg 0 0 4 0 2>&1 | grep Status > ${test_name}.3 &
sleep 1
${memcheck2} ./client_example ${test_dir}/zero_client_uuid.cfg 0 0 5 0 2>&1 | grep Error &> ${test_name}.4 &

wait

# NOP unless ${RUN_TESTS_UNDER_VALGRIND}=1
print_valgrind_error_summary

# Remove lingering semaphores, if any.
env QKD_SERVER_FORCE_SEM_UNLINK=1 ./server_example 1001 A

# Compare test output with reference log.
diff ${test_name}.1 ${test_dir}/reflog123 && \
diff ${test_name}.2 ${test_dir}/reflog123 && \
diff ${test_name}.3 ${test_dir}/reflog123 && \
diff ${test_name}.4 ${test_dir}/reflog4
