#!/bin/bash

# This example demonstrates what happens when keys are deleted due to TTF expiry.
#
# This may be a little trickier than expected.
#
# We set the TTF param to 3 (seconds) in the client's configuration file.
# The client issues a sequence of GET_KEY commands (see below).
# There is a 4 seconds delay prior to requesting the 4th key.
# Looking at the log file, we see
#   Index: 1
#   Status: success
#   Index: 2
#   Status: success
#   Index: 3
#   Status: success
#   Index: 5
#   Status: success
#   ...
#
# Success, four times in a row?  Where's the error due to TTF expiry?
# Check the index value!
# The server deleted the key with index=4 due to TTF expiry.
# Meanwhile, the client keeps sending GET_KEY requests with a zero index,
# essentially saying "give me the next available key, and tell me what
# index it has".  That's why the server correctly responds with "here
# is key with index 5".

test_dir=`dirname ${BASH_SOURCE[0]}`
test_name="ttf_expired"

# valgrind helpers (active if ${RUN_TESTS_UNDER_VALGRIND}=1)
. ${test_dir}/../../.valgrind

# Remove lingering semaphores, if any.
env QKD_SERVER_FORCE_SEM_UNLINK=1 ./server_example 1001 A

# The actual test starts here.
timeout 12s ${memcheck1} ./server_example 1001 A &
sleep 1
${memcheck2} ./client_example ${test_dir}/three_sec_ttf.cfg 1 0 1 0 1 0 4 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 | egrep "(Index|Status)" > ${test_name}.A 2>&1

wait

# NOP unless ${RUN_TESTS_UNDER_VALGRIND}=1
print_valgrind_error_summary

# Remove lingering semaphores, if any.
env QKD_SERVER_FORCE_SEM_UNLINK=1 ./server_example 1001 A

# Check results of both parts of the test.
diff ${test_name}.A ${test_dir}/reflog
