#include <assert.h>
#include <string.h>
#include <stdio.h>

#include "qkd/interfaces.h"
#include "qkd/copy.h"
#include "qkd/utils.h"

const version_t QKD_004_VERSION = {2, 1, 1};
const version_t QKD_004_PROTOCOL_VERSION = {1, 0, 0};

void (*qkd_open_connect)(qkd_uri_t destination_kme_id, qkd_uri_t source_sae_id,
                         qkd_uri_t destination_sae_id, qkd_qos_t *qos,
                         uuid_t *key_stream_id, qkd_metadata_t *metadata,
                         qkd_status_t *status, void *app_context_data) = 0;
void (*qkd_get_key)(uuid_t key_stream_id, uint32_t *index,
                    qkd_key_buffer_t *key_buffer, qkd_metadata_t *metadata,
                    qkd_status_t *status, void *app_context_data) = 0;
void (*qkd_close)(uuid_t key_stream_id, qkd_status_t *status,
                  void *app_context_data) = 0;

qkd_open_connect_response_t
qkd_open_connect_struct(qkd_open_connect_request_t open_connect_request,
                        void *app_context_data) {
    assert(app_context_data != NULL);
    qkd_open_connect_response_t open_connect_response = {};

    // In-out params
    qkd_copy_qos(&open_connect_response.qos, &open_connect_request.qos);
    memcpy(&open_connect_response.key_stream_id,
           &open_connect_request.key_stream_id, sizeof(uuid_t));

    assert(qkd_open_connect != NULL);
    qkd_open_connect(
        open_connect_request.destination_kme_id,
        open_connect_request.source_sae_id,
        open_connect_request.destination_sae_id, &open_connect_response.qos,
        &open_connect_response.key_stream_id, &open_connect_response.metadata,
        &open_connect_response.status, app_context_data);

#ifdef QKD_LOG_DEBUG
    fprintf(stdout, "[INFO] ");
    qkd_fprintf_oc_request(stdout, open_connect_request);
    fprintf(stdout, "\n[INFO] ");
    qkd_fprintf_oc_response(stdout, open_connect_response);
    fprintf(stdout, "\n");
#endif

    return open_connect_response;
}

qkd_get_key_response_t qkd_get_key_struct(qkd_get_key_request_t get_key_request,
                                          void *app_context_data) {
    assert(app_context_data != NULL);
    qkd_get_key_response_t get_key_response = {};

    // In-out params
    get_key_response.index = get_key_request.index;
    memcpy(&get_key_response.metadata, &get_key_request.metadata,
           sizeof(qkd_metadata_t));

    assert(qkd_get_key != NULL);
    qkd_get_key(get_key_request.key_stream_id, &get_key_response.index,
                &get_key_response.key_buffer, &get_key_response.metadata,
                &get_key_response.status, app_context_data);

#ifdef QKD_LOG_DEBUG
    fprintf(stdout, "[INFO] ");
    qkd_fprintf_gk_request(stdout, get_key_request);
    fprintf(stdout, "\n[INFO] ");
    qkd_fprintf_gk_response(stdout, get_key_response);
    fprintf(stdout, "\n");
#endif

    return get_key_response;
}

qkd_close_response_t qkd_close_struct(qkd_close_request_t close_request,
                                      void *app_context_data) {
    assert(app_context_data != NULL);
    qkd_close_response_t close_response = {};

    assert(qkd_close != NULL);
    qkd_close(close_request.key_stream_id, &close_response.status,
              app_context_data);

#ifdef QKD_LOG_DEBUG
    fprintf(stdout, "[INFO] ");
    qkd_fprintf_cl_request(stdout, close_request);
    fprintf(stdout, "\n[INFO] ");
    qkd_fprintf_cl_response(stdout, close_response);
    fprintf(stdout, "\n");
#endif

    return close_response;
}
