#include <stdio.h>

#include "qkd/cfg_parser.h"
#include "qkd/models.h"

/**
 * Unit test for configuration parser.
 * 
 * See cfg_parser_test.reflog for the expected output.
*/

#define NUM_CFG_FILES 2
static const char* cfg_filenames[NUM_CFG_FILES] = { "cfg.dat", "cfg2.dat" };

#define CFG_NUM_KEYWORDS 4
/**
 * Table of (optional) configuration file entries.
 * - port_number = the server's port number
 * - max_num_KSIDs = maximum number of KSIDs
 * - per_KSID_RB_size = per-KSID ringbuffer size in byte
 * - keygen_rate = key generation rate in byte per seconds
 */
static cfg_kv_pair_t cfg_table[CFG_NUM_KEYWORDS+1] = {
    {"port_number", NULL},
    {"max_num_KSIDs", NULL},
    {"per_KSID_RB_size", NULL},
    {"keygen_rate", NULL},
    {NULL, NULL} // the end-marker
};

int main(int argc, char *argv[]) {

    for (int i=0; i<NUM_CFG_FILES; i++) {
        const char* cfg_filename = cfg_filenames[i];

        // Configuration parameters and default values
        uint16_t port_number = QKD_004_DEFAULT_PORTNUMBER;
        long max_num_KSIDs = 4; // maximum number of KSIDs
        long per_KSID_RB_size = 512; // per-KSID ringbuffer size in byte
        long keygen_rate = 10; // key generation rate in byte per seconds

        printf("Reading configuration file %s ...\n", cfg_filename);
        if (cfg_from_file(cfg_table, cfg_filename) != 0) {
            fprintf(stderr, "Loading the configuration file failed\n");
        } else {
            long val;
            
            int status = cfg_get_long(cfg_table, "port_number", &val);
            if (status == 0) {
                if (val < 0 || val > 65535) {
                    fprintf(stderr, "Invalid port ID specified in configuration file "
                                    "(valid range: 0..65535), skipping entry ...\n");
                } else { port_number = val; }
            } else if (status != -1) { fprintf(stderr, "Skipping invalid port_number entry.\n"); } 

            status = cfg_get_long(cfg_table, "max_num_KSIDs", &val);
            if (status == 0) { max_num_KSIDs = val; }
            else if (status != -1) { fprintf(stderr, "Skipping invalid max_num_KSIDs entry.\n"); }
            
            status = cfg_get_long(cfg_table, "per_KSID_RB_size", &val);
            if (status == 0) { per_KSID_RB_size = val; }
            else if (status != -1) { fprintf(stderr, "Skipping invalid per_KSID_RB_size entry.\n"); }
            
            status = cfg_get_long(cfg_table, "keygen_rate", &val);
            if (status == 0) { keygen_rate = val; }
            else if (status != -1) { fprintf(stderr, "Skipping invalid keygen_rate entry.\n"); }
        }
    }
}