#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "examples/ringbuffer.h"

/**
 * Unit test for the ringbuffer type.
 *
 * See ringbuffer_test.reflog for the expected output.
 *
 * ATTN:
 * - This test has a fairly long runtime of more than 2 mins.
 * - This test is not 100% stable.  Due to timing variations
 *   (process scheduling), the outcome may not be exactly what
 *   is covered in the .reflog file.  More work is required to
 *   increase readability and stability of this test.
 */

int main(int argc, char* argv[]) {
    printf("Ringbuffer test\n");

#if 1
    {
        // Ringbuffer with zero TTF
        qkd_rb_t rb;
        qkd_rb_init(&rb, 10, 10000, 256, 0);

        sleep(2);

        unsigned index = 0;
        for (int i=0; i<16; i++) {
            if (qkd_rb_read(&rb, &index) == QKD_INDEX_READY) {
                printf("read succeeded at index %u\n", index);
                index++;
            } else {
                printf("read failed at index %u\n", index);
            }
        }
        qkd_rb_dtor(&rb);
    }
#endif

#if 1
    {
        // Ringbuffer with zero TTF
        qkd_rb_t rb;
        qkd_rb_init(&rb, 10, 64, 256, 0);

        unsigned index = 0;
        for (int i=0; i<16; i++) {
            if (qkd_rb_read(&rb, &index) == QKD_INDEX_READY) {
                printf("read succeeded at index %u\n", index);
                index++;
            } else {
                printf("read failed at index %u\n", index);
            }
            sleep(1);
        }

        index = 0;
        qkd_KSID_index_state_t res = qkd_rb_read(&rb, &index);
        printf("index=%d; value=%d\n", index, res);
        index = 1000;
        res = qkd_rb_read(&rb, &index);
        printf("index=%d; value=%d\n", index, res);
        sleep(6);
        index = 4;
        res = qkd_rb_read(&rb, &index);
        printf("index=%d; value=%d\n", index, res);
        index = 1;
        res = qkd_rb_read(&rb, &index);
        printf("index=%d; value=%d\n", index, res);

        index = 5;
        for (int i=0; i<64; i++) {
            if (qkd_rb_read(&rb, &index) == QKD_INDEX_READY) {
                printf("read succeeded at index %u\n", index);
                index++;
            } else {
                printf("read failed at index %u\n", index);
            }
            sleep(1);
        }
        qkd_rb_dtor(&rb);
    }
#endif

#if 1
    {
        // Ringbuffer with 2s TTF
        qkd_rb_t rb;
        qkd_rb_init(&rb, 10, 64, 256, 2);

        unsigned index = 0;
        for (int i=0; i<16; i++) {
            if (qkd_rb_read(&rb, &index) == QKD_INDEX_READY) {
                printf("read succeeded at index %u\n", index);
                index++;
            } else {
                printf("read failed at index %u\n", index);
            }
            sleep(1);
        }
        sleep(6);
        for (int i=0; i<8; i++) {
            if (qkd_rb_read(&rb, &index) == QKD_INDEX_READY) {
                printf("read succeeded at index %u\n", index);
                index++;
            } else {
                printf("read failed at index %u\n", index);
            }
            sleep(1);
        }
        for (int i=0; i<8; i++) {
            index = 0;
            if (qkd_rb_read(&rb, &index) == QKD_INDEX_READY) {
                printf("read succeeded at index %u\n", index);
            } else {
                printf("read failed at index %u\n", index);
            }
            sleep(1);
        }
        qkd_rb_dtor(&rb);
    }
#endif

    return 0;
}
